part of simple_billing;

class _SimpleBillingData {
  static Box _purchaseBox;
  static Box _subscriptionIAPItemsBox;
  static Box _productIAPItemsBox;
  static const String _boxPrefix = 'box_simple_billing';

  static Future init() async {
    final Directory appDocumentDir =
        await path_provider.getApplicationDocumentsDirectory();
    await Hive.initFlutter(appDocumentDir.path);
    _purchaseBox = await Hive.openBox('${_boxPrefix}_purchase');
    _subscriptionIAPItemsBox =
        await Hive.openBox('${_boxPrefix}_subscriptionIAPItems');
    _productIAPItemsBox = await Hive.openBox('${_boxPrefix}_productIAPItems');
  }

  static Set<PurchasedItem> get purchasedItems {
    return List<PurchasedItem>.from(_purchaseBox.values)
        .map((e) => PurchasedItem.fromJSON(
            json.decode(e as String) as Map<String, dynamic>))
        .toSet();
  }

  static Future savePurchases(Set<PurchasedItem> products) async {
    await _purchaseBox.clear();
    final Set<String> usedIds = {};
    for (final product in products) {
      if (usedIds.add(product.productId)) {
        await _purchaseBox.put(
            product.productId, json.encode(_productToJson(product)));
      }
    }
  }

  static List<IAPItem> get subscriptionItems {
    final Box box = _subscriptionIAPItemsBox;
    return List<Map<String, dynamic>>.from(box.values)
        .map((e) => IAPItem.fromJSON(e))
        .toList();
  }

  static List<IAPItem> get productItems {
    final Box box = _productIAPItemsBox;
    return List<Map<String, dynamic>>.from(box.values)
        .map((e) => IAPItem.fromJSON(e))
        .toList();
  }

  static List<IAPItem> get allIAPItems =>
      productItems..addAll(subscriptionItems);

  static Future saveProductsItem(List<IAPItem> items) async =>
      _saveIAPItems(items, isSubscription: false);

  static Future saveSubscriptionsItem(List<IAPItem> items) async =>
      _saveIAPItems(items, isSubscription: true);

  static Future _saveIAPItems(List<IAPItem> items,
      {@required bool isSubscription}) async {
    final Box box =
        isSubscription ? _subscriptionIAPItemsBox : _productIAPItemsBox;
    await box.clear();
    for (final item in items) {
      await box.put(item.productId, _iAPItemToJson(item));
    }
  }

  static Map<String, dynamic> _productToJson(PurchasedItem product) {
    return {
      'productId': product.productId,
      'transactionId': product.transactionId,
      'transactionDate': product.transactionDate?.millisecondsSinceEpoch,
      'transactionReceipt': product.transactionReceipt,
      'purchaseToken': product.purchaseToken,
      'orderId': product.orderId,

      /// android specific
      'dataAndroid': product.dataAndroid,
      'signatureAndroid': product.signatureAndroid,
      'autoRenewingAndroid': product.autoRenewingAndroid,
      'isAcknowledgedAndroid': product.isAcknowledgedAndroid,
      'purchaseStateAndroid': product.purchaseStateAndroid,
      'originalJsonAndroid': product.originalJsonAndroid,

      /// ios specific
      'originalTransactionDateIOS':
          product.originalTransactionDateIOS?.millisecondsSinceEpoch,
      'originalTransactionIdentifierIOS':
          product.originalTransactionIdentifierIOS,
      'transactionStateIOS': product.transactionStateIOS?.index,
    };
  }

  static Map<String, dynamic> _iAPItemToJson(IAPItem item) {
    return {
      'productId': item.productId,
      'price': item.price,
      'currency': item.currency,
      'localizedPrice': item.localizedPrice,
      'title': item.title,
      'description': item.description,
      'introductoryPrice': item.introductoryPrice,
      'subscriptionPeriodNumberIOS': item.subscriptionPeriodNumberIOS,
      'subscriptionPeriodUnitIOS': item.subscriptionPeriodUnitIOS,
      'introductoryPricePaymentModeIOS': item.introductoryPricePaymentModeIOS,
      'introductoryPriceNumberOfPeriodsIOS':
          item.introductoryPriceNumberOfPeriodsIOS,
      'introductoryPriceSubscriptionPeriodIOS':
          item.introductoryPriceSubscriptionPeriodIOS,
      'subscriptionPeriodAndroid': item.subscriptionPeriodAndroid,
      'introductoryPriceCyclesAndroid': item.introductoryPriceCyclesAndroid,
      'introductoryPricePeriodAndroid': item.introductoryPricePeriodAndroid,
      'freeTrialPeriodAndroid': item.freeTrialPeriodAndroid,
      'iconUrl': item.iconUrl,
      'originalJson': item.originalJson,
      'originalPrice': item.originalPrice,
    };
  }
}
